;;; Implementation of the Flight ADT.
;;; This code is included by the (flight) library defined in
;;; flight.sld, and should also work by itself in systems that
;;; support the older R5RS standard for Scheme plus SRFI 9 records.

;;; makeFlight : String String String NonNegInt NonNegInt -> Flight
;;; GIVEN: the name of a flight, the name of the airport from
;;;     which the flight departs, the name of the airport at
;;;     which the flight arrives, the time of departure in UTC,
;;;     and the time of arrival in UTC
;;; WHERE: the UTC times are less than 2400, their two low-order
;;;     digits are less than 60 and indicate minutes, and their
;;;     higher-order digits indicate hours
;;; RETURNS: a Flight value that encapsulates the given information
;;; EXAMPLE:
;;;     (makeFlight "United 448" "BOS" "DEN" 2003 0053)

(define (makeFlight name ap1 ap2 departs arrives)
  
  ;; Preconditions don't have to be checked as they are here,
  ;; but checking preconditions that are cheap to check makes
  ;; programs easier to debug.

  (if (and (string? name)
           (string? ap1)
           (string? ap2)
           (integer? departs)
           (integer? arrives)
           (<= 0 departs 2359)
           (<= 0 arrives 2359)
           (<= 0 (remainder departs 100) 59)
           (<= 0 (remainder arrives 100) 59))
      (makeFlight0 name ap1 ap2 departs arrives)
      (error "incorrect arguments passed to makeFlight: "
             name ap1 ap2 departs arrives)))

;;; flightName : Flight -> String
;;; GIVEN: a Flight
;;; RETURNS: the name of the Flight
;;; EXAMPLE:
;;;     (flightName (makeFlight "United 448" "BOS" "DEN" 2003 0053))
;;; =>  "United 448"

;;; departsFrom : Flight -> String
;;; GIVEN: a Flight
;;; RETURNS: the name of the airport from which the flight departs
;;; EXAMPLE:
;;;     (departsFrom (makeFlight "United 448" "BOS" "DEN" 2003 0053))
;;; =>  "BOS"

;;; arrivesAt : Flight -> String
;;; GIVEN: a Flight
;;; RETURNS: the name of the airport at which the flight arrives
;;; EXAMPLE:
;;;     (arrivesAt (makeFlight "United 448" "BOS" "DEN" 2003 0053))
;;; =>  "DEN"

;;; departureTime : Flight -> NonNegInt
;;; GIVEN: a Flight
;;; RETURNS: the time (in UTC, see above) at which the flight departs
;;; EXAMPLE:
;;;     (departureTime (makeFlight "United 448" "BOS" "DEN" 2003 0053))
;;; =>  2003

;;; arrivalTime : Flight -> NonNegInt
;;; GIVEN: a Flight
;;; RETURNS: the time (in UTC, see above) at which the flight arrives
;;; EXAMPLE:
;;;     (arrivalTime (makeFlight "United 448" "BOS" "DEN" 2003 0053))
;;; =>  0053

(define-record-type Flight
  (makeFlight0 name ap1 ap2 departs arrives)
  flight?
  (name    flightName)
  (ap1     departsFrom)
  (ap2     arrivesAt)
  (departs departureTime)
  (arrives arrivalTime))

;;; sameFlight : Flight Flight -> Boolean
;;; GIVEN: two flights
;;; RETURNS: true if and only if the two flights have the same
;;;     name, depart from the same airport, arrive at the same
;;;     airport, depart at the same time, and arrive at the same time
;;; EXAMPLES:
;;;
;;;     (sameFlight (makeFlight "United 448" "BOS" "DEN" 2003 0053)
;;;                 (makeFlight "United 448" "BOS" "DEN" 2003 0053))
;;; =>  true
;;;
;;;     (sameFlight (makeFlight "United 448" "BOS" "DEN" 2000 0055),
;;;                  makeFlight "United 448" "BOS" "DEN" 2003 0053))
;;; =>  false

(define (sameFlight flight1 flight2)
  (and (flight? flight1)
       (flight? flight2)
       (string=? (flightName flight1) (flightName flight2))
       (string=? (departsFrom flight1) (departsFrom flight2))
       (string=? (arrivesAt flight1) (arrivesAt flight2))
       (= (departureTime flight1) (departureTime flight2))
       (= (arrivalTime flight1) (arrivalTime flight2))))
